<?php
/** @var array $serverIps */
require_once '/var/www/html/core/network.ips.php';

// Directory for cached data
$cacheDir = '/var/www/html/storage/kline-cached/';

// Ensure the cache directory exists
if (!file_exists($cacheDir)) {
    mkdir($cacheDir, 0777, true);
}

// List of proxies
$proxies = [];
foreach ($display as $ip) {
    if (
        $ip !== '50.28.84.64' &&
        $ip !== '185.33.100.17' &&
        $ip !== '167.71.7.57' &&
        $ip !== '188.166.60.231'
    ) {
        $proxies[] = $ip;
    }
}

// Get a random member from the set
$proxy = $proxies[array_rand($proxies)];

header('Content-Type: application/json');

$symbol = $_GET['symbol'] ?? '';
$interval = $_GET['interval'] ?? '1d';
$limit = $_GET['limit'] ?? 2000;
$from_timestamp = intval($_GET['from_timestamp'] ?? '');
$debug = boolval($_GET['debug'] ?? '');
// min 1000 max 5000
$limit = min(5000, $limit);

$allowIntervals = [
    '1m', '3m', '5m', '15m', '30m', '1h', '2h', '4h', '6h', '8h', '12h', '1D', '3D', '1W', '1M'
];
$intervalMillisecond = [
    '1m' => 60000,
    '3m' => 180000,
    '5m' => 300000,
    '15m' => 900000,
    '30m' => 1800000,
    '1h' => 3600000,
    '2h' => 7200000,
    '4h' => 14400000,
    '6h' => 21600000,
    '8h' => 28800000,
    '12h' => 43200000,
    '1D' => 86400000,
    '3D' => 259200000,
    '1W' => 604800000,
    '1M' => 2592000000,
];
if (!in_array($interval, $allowIntervals)) {
    // Invalid interval
    echo json_encode(['error' => 'Invalid interval']);
    exit;
}

// Only use caching if from_timestamp is greater than 0
$useCache = $from_timestamp > 0;

// Cache key based on parameters
$cacheKey = $symbol . '_' . $interval . '_limit' . $limit . '_from' . $from_timestamp . '.json';
$cacheFile = $cacheDir . $cacheKey;

// Check if cache exists and is usable
if ($useCache && file_exists($cacheFile)) {
    // Serve cached data
    $cachedData = file_get_contents($cacheFile);
    echo $cachedData;
    exit;
}

// Binance API
$apiUrl = 'https://api.binance.com/api/v3/klines';

$params = [
    'symbol' => $symbol,
    'interval' => $interval,
    'limit' => $limit,
];

// Set proxy in header
header("X-Proxy-Server: $proxy");

// Select start point
$has_start_point = false;
if ($from_timestamp > 0)
{
    // Convert to milliseconds if it's in seconds
    if ($from_timestamp < 10000000000) {
        $from_timestamp *= 1000;
    }

    $params['startTime'] = $from_timestamp;
    $has_start_point = $from_timestamp;

    // Offset start time back 100 candles
    $params['startTime'] -= $intervalMillisecond[$interval] * 100;
}
else
{

    $_GET['file']="/var/www/html/storage/realtime-kline/".$symbol."-".$interval."-".($limit-100).".json";


    $candles=array();

    if(file_exists($_GET['file']))
    {
        $candles=json_decode(file_get_contents($_GET['file']),true);
    }
    ##/
}

//if(count($candles)==0)
if(!isset($_GET['file']))
{
    $url = $apiUrl . '?' . http_build_query($params);
    $url = "http://$proxy/bt/proxy.php?url=" . urlencode($url);

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    $output = curl_exec($ch);
    curl_close($ch);

    $candles = json_decode($output, true);
}

$now = time() * 1000;
$logs = [];
$max = 4;
do {
    // Check limit
    if (count($candles) < $limit) {
        // Another 1000 candles
        if ($has_start_point) {
            $params['startTime'] = end($candles)[0] + 1000;

            $closed = end($candles)[6];
            if ($closed > $now) {
                break;
            }
        } else {
            $params['startTime'] = $candles[0][0] - 1000;
            if ($params['startTime'] < 0) {
                unset($params['startTime']);
            }
        }

        $head_url = $apiUrl . '?' . http_build_query($params);
        $head_url = "http://$proxy/bt/proxy.php?url=" . urlencode($head_url);

        $logs[] = $head_url;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $head_url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $output = curl_exec($ch);
        curl_close($ch);

        $stack = json_decode($output, true);
        if (json_last_error() === JSON_ERROR_NONE || count($stack) > 0) {
            if ($has_start_point) {
                $candles = array_merge($candles, $stack);
            } else {
                $candles = array_merge($stack, $candles);
            }
        } else {
            break;
        }
    } else {
        break;
    }
    $max--;
    if ($max === 0) {
        break;
    }
} while (true);

// Save to cache only if from_timestamp is set and greater than 0
if ($useCache) {
    file_put_contents($cacheFile, json_encode($candles));
}

if ($debug) {
    // Compare each stick with the next one
    $errors = [];
    for ($i = 1; $i < count($candles); $i++) {
        $current = $candles[$i];
        $previous = $candles[$i - 1];
        $diff = $current[0] - $previous[0]; // Difference in minutes
        if ($diff !== $intervalMillisecond[$interval]) {
            $errors[] = [
                'time' => [
                    'current' => $current[0],
                    'previous' => $previous[0],
                ],
                'index' => $i,
                'diff' => $diff,
            ];
        }
    }

    echo json_encode([
        'count' => count($candles),
        'first' => $candles[0][0],
        'last' => $candles[count($candles) - 1][0],
        'logs' => $logs,
        'errors' => $errors,
    ]);
} else {
    echo json_encode(array_values($candles));
}
