<?php

session_start(['read_and_close' => true]);

require_once __DIR__ . '/_fun_.php';

$start = microtime(true);

$symbol = strtoupper($_GET['symbol'] ?? '');
$interval = $_GET['interval'] ?? '1h';
$limit = intval($_GET['limit'] ?? 2000);
$indicator = $_GET['indicator'] ?? 'klines';
$from = $_GET['from'] ?? '';
$debug = $_GET['debug'] ?? '';

// Supported intervals
$intervals = [
    '1m',
    '3m',
    '5m',
    '15m',
    '30m',
    '1h',
    '2h',
    '4h',
    '6h',
    '8h',
    '12h',
    '1D',
];

// Validate symbol
if (empty($symbol)) {
    header('Content-Type: application/json');
    echo json_encode(['error' => 'Symbol is required']);
    exit;
}

// Validate interval
if (!in_array($interval, $intervals)) {
    header('Content-Type: application/json');
    echo json_encode(['error' => 'Invalid interval']);
    exit;
}

//$allowIndicators = ['klines', 'rsi', 'tdsequential', 'ema', 'cmf', 'mfi', 'minmax', 'ma', 'roc', 'psar', 'supertrend', 'vwap', 'min', 'trix', 'vortex', 'bop', 'ppo', 'obv', 'cci', 'bband', 'ht_trendline'];
//if (!in_array($indicator, $allowIndicators)) {
//    // Set default indicator
//    $indicator = 'klines';
//}

$cache = false;

//$port = 9981; // Original version
$port = 9545; // New version
$url = "http://127.0.0.1:{$port}/{$indicator}/{$symbol}?interval={$interval}&limit={$limit}";

if (!empty($from)) {
    $url .= "&from_timestamp={$from}";

    // Estimate the latest timestamp based on the limit and interval
    $tick_size = parse_interval_to_seconds($interval);
    $tick_start = $from - ($from % $tick_size);

    // Get trick end based on limit
    $tick_end = $tick_start + ($limit * $tick_size);

    // Check $tick_end in between 30 days range
    $diff = time() - $tick_end;
    $days30 = 60 * 60 * 24 * 30;

    // If more than 30 days "CACHE" the data
    if ($diff > $days30) {
        $cache = true;
    }
}

// Check if the file is cached
$cached_file = $cache ? is_cached($url, $interval) : false;

// Payload
$payload = [];

if ($cached_file) {
    $raw = file_get_contents($cached_file);
    $payload = json_decode($raw, true);

    // Update metric
    $payload['metrics']['cached'] = true;
    $payload['metrics']['file_ages'] = time() - filemtime($cached_file);
} else {
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    $raw = curl_exec($ch);
    curl_close($ch);

    $json = json_decode($raw, true);
    if (json_last_error() !== JSON_ERROR_NONE || empty($json['data'])) {
        header('Content-Type: application/json');
        echo json_encode([
            'symbol' => $symbol,
            'interval' => $interval,
            'limit' => $limit,
            'metrics' => [
                'proxy_server' => $json['proxy_server'] ?? 'n/a',
                'kline_time_ms' => $json['kline_time'] ?? 0,
                'indicator_time_ms' => $json['indicator_time'] ?? 0,
                'proxies' => $json['proxies'] ?? [],
                'url' => $url,
            ],
            'data' => [],
            'reason' => $json
        ]);
        file_put_contents('/root/indicators-error.log', date('c') . ' ' . json_encode($json) . PHP_EOL, FILE_APPEND);
        exit;
    }

    $values = $json['data'];
    if (!empty($from)) {
        $from = intval($from);
        $values = array_filter($values, function ($value) use ($from) {
            return $value['timestamp'] >= $from;
        });
        $values = array_values($values);

        if (count($values) > $limit) {
            // keep 0 - $limit and trim the rest
            $values = array_slice($values, 0, $limit);
        }
    } else if (count($values) > $limit) {
        $values = array_slice($values, count($values) - $limit);
    }

    // Payload
    $payload = [
        'symbol' => $symbol,
        'interval' => $interval,
        'limit' => $limit,
        'total' => count($values),
        'metrics' => [
            'cached' => false,
            'proxy_server' => $json['proxy_server'] ?? 'n/a',
            'kline_time_ms' => $json['kline_time'] ?? 0,
            'indicator_time_ms' => $json['indicator_time'] ?? 0,
            'proxies' => $json['proxies'] ?? [],
        ],
        'data' => $values,
    ];

    if ($cache) {
        $payload['metrics']['cached'] = true;
        $payload['metrics']['file_ages'] = 0;

        // Cache the new data
        $cache_folder = "/var/www/html/storage/cache-indicators/";
        $filename = get_url_hash($url);
        file_put_contents($cache_folder . $filename, json_encode($payload));
    }
}

$end = microtime(true);

// processing time ms
$payload['metrics']['processing_time_ms'] = round(($end - $start) * 1000, 2);

header('Content-Type: application/json');
echo json_encode($payload);